#!/usr/bin/python

class FilterModule(object):
    '''Various utilities for manipulating NetBox data'''
    def __init__(self):
        self.virtual_iface_types = ('bridge', 'lag', 'virtual')

    def filters(self):
        return {
            'compact_numlist': self.compact_numlist,
            'iface_real': self.iface_real,
            'iface_peer': self.iface_peer,
            'iface_vlans': self.iface_vlans
        }

    def compact_numlist(self, nums, sort=True, delimiter=',', range_delimiter='-'):
        '''Transform [1,2,3,5,7,8,9] into "1-3,5,7-9"'''
        if sort:
            nums = sorted(nums)
        i = 0
        spans = []
        while i < len(nums):
            j = i + 1
            while j < len(nums) and nums[j]-nums[i] == j-i:
                j += 1
            spans += [f'{nums[i]}{range_delimiter}{nums[j-1]}' if j > i+1 else f'{nums[i]}']
            i = j
        return delimiter.join(spans)

    def iface_real(self, interfaces):
        '''Return only non-virtual interfaces'''
        for iface in interfaces:
            if iface.get('type', {}).get('value') not in self.virtual_iface_types:
                yield iface

    def iface_peer(self, interfaces):
        '''Return the name of the device connected to this interface'''
        for iface in interfaces:
            endpoint = (iface.get('connected_endpoints') or [{}])[0]
            if device := endpoint.get('device', {}).get('name'):
                yield device
            else:
                yield None

    def iface_vlans(self, interfaces):
        '''Returns a list of allowed VLANs for each interface'''
        for iface in interfaces:
            match iface.get('mode', {}).get('value'):
                case 'access':
                    yield [iface.get('untagged_vlan')] if 'untagged_vlan' in iface else []
                case 'tagged':
                    yield iface.get('tagged_vlans', [])
                case _:
                    yield []
